﻿// --------------------------------------------------------------------------------
// File: MainWindowViewModel.cs
//
// Copyright © 2019 APG Cash Drawer Ltd. 
//
// No part of this document may be reproduced or transmitted 
// by any form, or in any means including photocopying, and 
// recording, without the prior permission of APG Cash Drawer Ltd.
//
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
// OR OTHER DEALINGS IN THE SOFTWARE.
//-------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO.Ports;
using System.Runtime.CompilerServices;
using System.Windows.Input;

namespace APG_EMEA_USB_Sample
{
    class MainWindowViewModel : INotifyPropertyChanged
    {
        ApgEmeaUsbDrawer _usbDrawer = new ApgEmeaUsbDrawer();        //Instance of the usb drawer

        public List<string> PortNames
        {
            get
            {
                List<string> tempList = new List<string>();

                //Populate list with COM ports
                foreach (string s in SerialPort.GetPortNames())
                {
                    tempList.Add(s);
                }

                return tempList;
            }
        }
        public string DrawerStatus
        {
            get
            {
                if (_usbDrawer.IsOpen == true)
                    return "OPEN";
                if (_usbDrawer.IsOpen == false)
                    return "CLOSED";

                return "UNKNOWN";
            }

            private set { }
        }
        public string ConnectedButtonText
        {
            get
            {
                if(_usbDrawer.IsConnected)
                {
                    return "Disconnect";
                }
                else
                {
                    return "Connect";
                }
            }

            private set { }
        }
        public bool IsConnected { get { return _usbDrawer.IsConnected; } }

        public ICommand OnOpenClickedCommand { get; private set; }
        public ICommand OnGetStatusClickedCommand { get; private set; }
        public ICommand OnClearStatusClickedCommand { get; private set; }
        public ICommand OnConnectClickedCommand { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        public MainWindowViewModel()
        {
            OnOpenClickedCommand        = new DelegateCommand(_openDrawer);
            OnGetStatusClickedCommand   = new DelegateCommand(_getStatus);
            OnConnectClickedCommand     = new DelegateCommand(_connectDrawer);
            OnClearStatusClickedCommand = new DelegateCommand(_clearStatus);

            //Notify change of drawer status.
            _usbDrawer.DrawerStatusChanged += (e,a) => _notifyPropertyChanged(nameof(DrawerStatus));
            _usbDrawer.DrawerConnectionStatusChanged += (e, a) => 
            {
                _notifyPropertyChanged(nameof(IsConnected));
                _notifyPropertyChanged(nameof(ConnectedButtonText));
            };
        }

        void _notifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        void _openDrawer(object parameter)
        {
            _usbDrawer.Open();
        }

        void _getStatus(object parameter)
        {
            _usbDrawer.GetStatus();
        }

        void _connectDrawer(object parameter)
        {
            if (IsConnected)
            {
                _usbDrawer.Disconnect();
            }
            else
            {
                if (parameter != null)
                    _usbDrawer.Connect((string)parameter);
            }
        }

        void _clearStatus(object parameter)
        {
            _usbDrawer.ClearStatus();
            _notifyPropertyChanged(nameof(DrawerStatus));
        }
    }

}
